#ifndef __CSHADE_H
#define __CSHADE_H

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************

	File        :   cshade.h

	Description :   CyberShades LCD glasses support definitions

	Version     :   1.01

	Comments    :   Read file cshade.txt for more details.

	Copyright   :   1995 Chinon America, Inc. All rights reserved.

					VGA frequency "flicker fixer" code
courtesy of
					Joerg Meyer, Dep. of Computer Sciences
					University of Kaiserslautern,  Germany.

					The code contained therein is copyrighted
by
					Chinon America, Inc., which reserves its
rights for
					distribution and use of this product.
You
					should first consult with Chinon America
before
					modifying it, distributing it or copying
some or all of
					its content.

*******************************************************************************/

#define _CSHADE_VERSION       (0x101)

/*******************************************************************************
**
**  CyberShades device masks
*/

	#define PCP_LEFTMASK        (0xCE)
	#define PCP_RIGHTMASK       (0xCA)

	#define PCS_LEFTMASK        (0x03)
	#define PCS_RIGHTMASK       (0x01)

	#define PC3D_LEFTMASK       (0x01)
	#define PC3D_RIGHTMASK      (0x03)

	#define PCVR_COM_LEFTMASK       (0x03)
	#define PCVR_COM_RIGHTMASK      (0x01)

	#define PCVR_LPT_LEFTMASK       (0x04)
	#define PCVR_LPT_RIGHTMASK      (0x08)

	#define SS1_COM_LEFTMASK        (0x03)
	#define SS1_COM_RIGHTMASK       (0x00)

	#define SS1_LPT_LEFTMASK        (0x04)
	#define SS1_LPT_RIGHTMASK       (0x00)


/*******************************************************************************
**
**  i/o Port-related definitions
**  ============================
*/
	typedef enum {
		Disabled = -1,
		Com1, Com2, Com3, Com4,
		Lpt1, Lpt2, Lpt3, Lpt4,
	} StandardPort;

/*
**  Functions
**  =========
**
**  To get port address
*/
	unsigned int PortAddress (StandardPort p);

/*  To read form a port
*/
	char PortRead (StandardPort p);

/*  To write to a port
*/
	void PortWrite (StandardPort p, char byte);

/*  To determine existence of a port
*/
	#define PortExists(p)       (PortAddress(p) != 0)

/*  To determine type of port
*/
	#define IsLptPort(p)        (((p) >= Lpt1) && ((p) <= Lpt4))
	#define IsComPort(p)        (((p) >= Com1) && ((p) <= Com4))


/*******************************************************************************
**
**  LCD glasses specific definitions
**  ================================
**
**  devices
**  ============
*/
	typedef enum {
		PCP,
		PCS,
		PC3D,
		PCVR,
		SS1,        /*  Stereo Space model 1    */
	} _CyberShadesDevice;

/*  Valid commands for glasses
**  ==========================
*/
	typedef enum {
		Off = -1,                       /*  turn glasses OFF
*/
		Left,                           /*  clear left side (obscur left)
*/
		Right,                          /*  clear right side
*/
		Flip,                           /*  toggle  sides
*/
		Switch,                         /*  invert left/right side
*/
	} GlassesCommand;

	/*  Invert command :    Invert switches left mask and right mask
							Use cautiously !
*/

/*  Glasses device data
**  ===================
*/
	typedef struct {
		StandardPort    Port;           /*  port the device is attached
to  */
		unsigned char   LeftMask;       /*  left mask value
*/
		unsigned char   RightMask;      /*  right mask value
*/
		unsigned char   OffMask;        /*  off value
*/
		GlassesCommand  State;          /*  current status (read-only)
*/
		int             Reversed;       /*  invert status  (read-only)
*/
	} GlassesData;

/*
**  Functions
**  =========
**
**  To initialize glasses data with a CyberShades device
*/
	int InitCyberShadesGlasses (GlassesData* data, _CyberShadesDevice device,
							StandardPort port);

/*  To initialize with specific data
*/
	int InitCustomGlasses (GlassesData* data, StandardPort port,
							char leftMask, char
rightMask, char OffMask);

/*  To send commands to the glasses
*/
	void TellGlasses (GlassesData* data, GlassesCommand command);


/*******************************************************************************
**
**  Timer installation
**  ==================
**
**  user-defined VGA flipping procedure type definition
*/
	typedef void (VGAFlipProc)(GlassesCommand page);

/*  To Install timer, returns 0 if error
**
**  userProc is your VGA page flipping routine
**  firstSide is the side you want to start on
*/
	int StartGlasses (GlassesData* data, VGAFlipProc* userProc,
						GlassesCommand firstSide);

/*  To install timer for multiple glasses, returns 0 if error
*/
	int StartMultipleGlasses (GlassesData* data, int nGlasses,
								VGAFlipProc*
userProc, GlassesCommand firstSide);

/*  To uninstall timer
*/
	void StopGlasses ();


/*******************************************************************************
**
**  VGA frequency control.
**	======================
**
**	return TRUE if successful.
*/
	int IncreaseVGAFreq ();


#ifdef __cplusplus
}
#endif

#endif /***********************************************************************/


