/* dramgfx.c */
/* Created 2/03/95 */
/* Copyright 1995 John Schultz */
/* All Rights Reserved */

#include <conio.h>

#include "imath.h"
#include "vector.h"
#include "gfx.h"
#include "vgfx.h"
#include "vtimer.h"

#include "lcdcall.h"

#define SC_INDEX 0x03C4

// Video memory page 0
#define VMPAGE0 0xa0000

static char * dramPage[4];
static long drawPage = 0;

void d_setDramPages(char * mem) {
  dramPage[0] = mem;
  dramPage[1] = &mem[320*200];
  dramPage[2] = dramPage[0];
  dramPage[3] = dramPage[1];
} // d_setDramPages

void d_setDisplayMode(displayMode dm) {
  _disable();
  vdata.view = 0;
  gfx.advancePage = 0;
  gfx.dispMode = dm;
  if (gfx.dispMode == DISPLAY_STEREO) {
    gfx.viewPage = 0;
    gfx.drawPage = 2;
  } else {
    gfx.viewPage = 0;
    gfx.drawPage = 1;
  } // if
  gfx.numPages   = 4;
  drawPage = gfx.drawPage;
  _enable();
} // d_setDisplayMode

void d_clearPage(long color) {
register unsigned long * p = (unsigned long *)dramPage[drawPage];
long i;
  color &= 0xff;
  color |= color<<8;
  color |= color<<16;
  for (i=0; i < 320*200/4/4; i++) {
    *p++ = color;
    *p++ = color;
    *p++ = color;
    *p++ = color;
  } // for
} // d_clearPage

void d_copyPage(long destPage,char * srcPage) {
unsigned long * destAddr;
long i,j;
  outp(SC_INDEX,2); // Set up (point to) plane mask register
  for (j=0; j < 4; j++) {
    outp(SC_INDEX+1,1<<j); // Point to specific plane
    destAddr = (unsigned long *)vmpage[destPage];
    for (i=0; i < 320*200; i += 16) {
      long ij = i+j;
      *destAddr++ = srcPage[ij] | (srcPage[ij+4]<<8) | (srcPage[ij+8]<<16) | (srcPage[ij+12]<<24);
    } // for
  } // for
} // d_copyPage

void d_copyPagesStereo200(long destPage,char * srcPageL,char * srcPageR) {
  d_copyPage(destPage,  srcPageL);
  d_copyPage(destPage+1,srcPageR);
} // d_copyPagesStereo200

void d_setPixel(vec2 * p,long color) {
char * pageAddr = dramPage[drawPage];
  pageAddr[p->y*320+p->x] = color;
} // d_setPixel

void d_drawLine(vec2 * l,long color) {
long x1 = l[0].x;
long y1 = l[0].y;
long x2 = l[1].x;
long y2 = l[1].y;
long d,x,y,ax,ay,sx,sy,dx,dy;
register char * addr = dramPage[drawPage];
  dx = x2-x1; ax = abs(dx)<<1; sx = sgn(dx);
  dy = y2-y1; ay = abs(dy)<<1; sy = sgn(dy);
  x = x1;
  y = y1;
  if (ax > ay) { /* x dominant */
    d = ay - (ax>>1);
    while (1) {
      addr[y*320+x] = color;
      if (x == x2) return;
      if (d >= 0) {
      y += sy; d -= ax;
	  } // if
	  x += sx; d += ay;
	} // while
    } else {       /* y dominant */
      d = ax - (ay>>1);
      while (1) {
        addr[y*320+x] = color;
        if (y == y2) return;
        if (d >= 0) {
        x += sx; d -= ay;
      } // if
      y += sy; d += ax;
    } // while
  } // if
} // d_drawLine

void d_fillRect(vec2 * extent,long color) {
long x1 = extent[0].x;
long y1 = extent[0].y;
long x2 = extent[1].x;
long y2 = extent[1].y;
register char * addr = dramPage[drawPage];
long x,y,temp;
  if (x1 > x2) {temp = x1; x1 = x2; x2 = temp;}
  if (y1 > y2) {temp = y1; x1 = y2; y2 = temp;}
  for (x=x1; x <= x2; x++) {
    for (y=y1; y <= y2; y++) {
      addr[y*320+x] = color;
    } // for
  } // for
} // d_fillRect

void d_waitView(void) {
} // d_waitView

void d_updateView(void) {
  while (gfx.advancePage);
  if (gfx.dispMode == DISPLAY_STEREO) {
    d_copyPagesStereo200(gfx.drawPage,dramPage[0],dramPage[1]);
    if ((gfx.drawPage += 2) >= gfx.numPages) gfx.drawPage = 0;
  } else {
    d_copyPage(gfx.drawPage,dramPage[0]);
    if ((gfx.drawPage += 1) >= gfx.numPages) gfx.drawPage = 0;
  } // if
  drawPage = gfx.drawPage;
  vdata.simFrames++;
  gfx.advancePage = 1;
} // d_updateView

void d_setViewPage(long page) {
  setViewPageAddr(vmpage[page]);
} // d_setViewPage

void d_setDrawPage(gfxDrawLR dlr) {
  if (dlr == DRAW_LEFT) {
    drawPage = gfx.drawPage;
  } else {
    drawPage = gfx.drawPage+1;
  } // if
} // d_setDrawPage

// ************************ LCDBIOS routines **************************

void d_setDisplayModeLcdbios(displayMode dm) {
  _disable();
  vdata.view = 0;
  gfx.advancePage = 0;
  gfx.dispMode = dm;
  if (gfx.dispMode == DISPLAY_STEREO) {
    gfx.viewPage = 0;
    gfx.drawPage = 2;
  } else {
    gfx.viewPage = 0;
    gfx.drawPage = 1;
  } // if
  gfx.numPages   = 4;
  drawPage = gfx.drawPage;
  _enable();
} // d_setDisplayModeLcdbios

void d_waitViewLcdbios(void) {
} // d_waitViewLcdbios

void d_updateViewLcdbios(void) {

  LCDBiosPageWait();

  if (gfx.dispMode == DISPLAY_STEREO) {
    d_copyPagesStereo200(gfx.drawPage,dramPage[0],dramPage[1]);
    if ((gfx.drawPage += 2) >= gfx.numPages) gfx.drawPage = 0;
  } else {
    d_copyPage(gfx.drawPage,dramPage[0]);
    if ((gfx.drawPage += 1) >= gfx.numPages) gfx.drawPage = 0;
  } // if
  drawPage = gfx.drawPage;
  vdata.simFrames++;

  LCDBiosAdvancePage();

} // d_updateViewLcdbios

void d_setViewPageLcdbios(long page) {
} // d_setViewPageLcdbios

void d_setDrawPageLcdbios(gfxDrawLR dlr) {
  if (dlr == DRAW_LEFT) {
    drawPage = gfx.drawPage;
  } else {
    drawPage = gfx.drawPage+1;
  } // if
} // d_setDrawPageLcdbios

// dramgfx.c

