/* grid.c */
/* Created 1/16/95 */
/* John Schultz */

#include <stdlib.h>
#include "imath.h"
#include "vector.h"
#include "gfx.h"
#include "vgfx.h"
#include "dramgfx.h"

typedef struct {
  vec3 v[2];
	vec2 p[2];
	coord zfar;
	long color;
} line3;

static long numLines = 0;
static line3 * l = 0;
static line3 ** sl = 0;

#define GA_X 1
#define GA_Y 2
#define GA_Z 3

static void makeGrid(line3 * l,long count,coord width,long axis,long rot90,
                     long r,long g,long b) {
matrix33 m;
vec3 rotAxis;
coord inc,hwidth,offset;
long cr,cg,cb;
long bright,db;
long i;

  if (count < 2) return;

  switch (axis) {
    case -GA_X:
      SETVEC(rotAxis,0,0,-TOFIXED(1.));
      matrix33axis(&m,&rotAxis,RAD090DEG);
      break;
    case GA_X:
      SETVEC(rotAxis,0,0,TOFIXED(1.));
      matrix33axis(&m,&rotAxis,RAD090DEG);
      break;
    case -GA_Y:
      SETVEC(rotAxis,0,0,TOFIXED(1.));
      matrix33axis(&m,&rotAxis,RAD180DEG);
      break;
    case GA_Y:
      matrix33unitize(&m);
      break;
    case -GA_Z:
      SETVEC(rotAxis,-TOFIXED(1.),0,0);
      matrix33axis(&m,&rotAxis,RAD090DEG);
      break;
    case GA_Z:
      SETVEC(rotAxis,TOFIXED(1.),0,0);
      matrix33axis(&m,&rotAxis,RAD090DEG);
      break;
  } // switch

  if (rot90) {
    matrix33 t1,t2;
    SETVEC(rotAxis,0,TOFIXED(1.),0);
    matrix33axis(&t1,&rotAxis,RAD090DEG);
    matrixMul33(&t2,&t1,&m);
    m = t2;
  } // if

  inc = width/(count-1);
  hwidth = width/2;
  offset = -hwidth;

  db = DIV(1,(count+1)/2);
  bright = 0;

  for (i=0; i < count; i++) {
    vec3 p0,p1;
    vec3 rp0,rp1;
    SETVEC(p0,offset,hwidth,hwidth);
    SETVEC(p1,p0.x,p0.y,-p0.z);
    rotate33(&rp0,&m,&p0);
    rotate33(&rp1,&m,&p1);
    l[i].v[0] = rp0;
    l[i].v[1] = rp1;
    bright += db;
    if (i == (count+1)/2) db = -db;
    cr = MUL(r,bright);
    cg = MUL(g,bright);
    cb = MUL(b,bright);
    CLAMPMAX(cr,0xff); CLAMPMAX(cg,0xff); CLAMPMAX(cb,0xff);
    l[i].color = DCOLOR(cr,cg,cb);
    offset += inc;
  } // for

} // makeGrid

long initGrid(long linesPerFace,coord width) {

  if (linesPerFace < 2) return 0;

  numLines = linesPerFace*12;

  if (l) free(l);
  l = (line3 *)calloc(1,sizeof(*l)*numLines);
  if (!l) return 0;

  if (sl) free(sl);
  sl = (line3 **)calloc(1,sizeof(sl)*numLines);
	if (!sl) return 0;

#if 1
  makeGrid(&l[linesPerFace *  0],linesPerFace,width, GA_X,0,0xff,0,0);
  makeGrid(&l[linesPerFace *  1],linesPerFace,width, GA_Z,1,0xff,0,0);
  makeGrid(&l[linesPerFace *  2],linesPerFace,width,-GA_X,0,0xff,0,0);
  makeGrid(&l[linesPerFace *  3],linesPerFace,width,-GA_Z,1,0xff,0,0);
#endif

#if 1
  makeGrid(&l[linesPerFace *  4],linesPerFace,width, GA_Y,0,0,0xff,0);
  makeGrid(&l[linesPerFace *  5],linesPerFace,width, GA_Y,1,0,0xff,0);
  makeGrid(&l[linesPerFace *  6],linesPerFace,width,-GA_Y,0,0,0xff,0);
  makeGrid(&l[linesPerFace *  7],linesPerFace,width,-GA_Y,1,0,0xff,0);
#endif

#if 1
  makeGrid(&l[linesPerFace *  8],linesPerFace,width, GA_Z,0,0,0,0xff);
  makeGrid(&l[linesPerFace *  9],linesPerFace,width, GA_X,1,0,0,0xff);
  makeGrid(&l[linesPerFace * 10],linesPerFace,width,-GA_Z,0,0,0,0xff);
  makeGrid(&l[linesPerFace * 11],linesPerFace,width,-GA_X,1,0,0,0xff);
#endif

  return 1;

} // initGrid

void deInitGrid(void) {
  if (l) free(l);
  l = 0;
  if (sl) free(sl);
	sl = 0;
} // deInitGrid

typedef int (*qcmp)(const void *,const void *);

int zcomp(line3 ** a,line3 ** b) {
  if ((*a)->zfar < (*b)->zfar)
    return 1;
  else
    return -1;
} // zcomp

void sortLines(line3 ** table,int count) {
  qsort((char *)table,count,sizeof(*table),(qcmp)zcomp);
} // sortLines

long count;

void transformGrid(matrix34 * m) {
long i;

  count=0;

  for (i=0; i < numLines; i++) {
    vec3 in[2];
    rotate34(&in[0],m,&l[i].v[0]);
    rotate34(&in[1],m,&l[i].v[1]);
    if (projClipLineStereo(&l[i].p,in)) {
      l[i].zfar = MAX(in[0].z,in[1].z);
//			l[i].zfar = (in[0].z+in[1].z)/2;
			sl[count++] = &l[i];
		} // if
	} // for

	sortLines(sl,count);

} // transformGrid

void drawGrid(matrix34 * m) {
long i;
	transformGrid(m);
	for (i=0; i < count; i++) {
    (*gfx.drawLine)(sl[i]->p,sl[i]->color);
  } // for
} // drawGrid

/* grid.c */

