// gridemo.c
// Created 1/19/95
// Modified 10/1/95
// Copyright 1995 John Schultz
// All Rights Reserved

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <ctype.h>
#include "imath.h"
#include "vector.h"
#include "clip.h"
#include "gfx.h"
#include "vgfx.h"
#include "dramgfx.h"
#include "vfont.h"
#include "vtimer.h"
#include "joystick.h"

#include "grid.h"
#include "gridemo.h"

#include "lcdcall.h"

#define MY_JOY_MAX_READ 4096

#define SOUND_DRIVER_CALL_RATE 120
#define VRR_MULTIPLIER 2
#define TIMER_ADVANCE 7.16 // 7.16
#define TIMER_ADVANCE_PERCENT (TIMER_ADVANCE/(float)VRR_MULTIPLIER)

enum {SM_VRAM=0,SM_DRAM=1,SM_LCDB=2};

// msep = (VT) View Translation by moving the camera.
// sep  = (CP) screen separation (moves the Center of Projection on the screen).

static stereoSMC smcl,smcr;
static coord zoom,msep,sep;
static long stereoMode = SM_VRAM;
static vfont vf;
static vec3 vcenter;
static long background = 1;
static joyInfo ji;
static long joyReadOk = 0;
static long showJoyInfo = 1;
static long autoRot = 1;

// Compute frame rate every SIM_COUNT frames
#define SIM_COUNT 30

static float getFrameRate(void) {
static long simFrames=0,t1=0;
static float frameRate;

  if (++simFrames == SIM_COUNT) {
    float dt;
    dt = getMicroSecs() - t1;
    dt = dt / 1e6; // convert from micros to secs
    if (dt != 0.) {
      frameRate = (float)SIM_COUNT/dt;
    } else {
      frameRate = 0.;
    } // if
    resetMicroSecs();
    t1 = getMicroSecs();
    simFrames = 0;
  } // if

  return frameRate;

} // getFrameRate

static void displayInfo(matrix34 * m,vec3 * center,float frameRate) {
vec3 c;
long color;
char s[128];
char mode;

  if (background) color = DCOLOR(0xff,0xff,0xff);
  else color = DCOLOR(0x00,0xaf,0x0f);
  sprintf(s,"vt %d cp %d zm %d",msep,sep,zoom);
  drawVString(&vf,s,m,center,VF_ALIGN_CENTER,color);

  c = *center;

  if (showJoyInfo) {

    if (!joyReadOk) {
      c.y -= vf.interLineSpace+vf.height+1;
      drawVString(&vf,"Press J to disable joystick",m,&c,VF_ALIGN_CENTER,color);
    } // if

    sprintf(s,"B x %5d y %5d b1 %d b2 %d",
            ji.js[1].x,
            ji.js[1].y,
            ji.js[1].button[0],
            ji.js[1].button[1]);
    c.y -= vf.interLineSpace+vf.height+1;
    drawVString(&vf,s,m,&c,VF_ALIGN_CENTER,color);

    sprintf(s,"A x %5d y %5d b1 %d b2 %d",
            ji.js[0].x,
            ji.js[0].y,
            ji.js[0].button[0],
            ji.js[0].button[1]);
    c.y -= vf.interLineSpace+vf.height+1;
    drawVString(&vf,s,m,&c,VF_ALIGN_CENTER,color);

  } // if

  switch (stereoMode) {
    case SM_VRAM: mode = 'V'; break;
    case SM_DRAM: mode = 'D'; break;
    case SM_LCDB: mode = 'L'; break;
  } // switch

  c.y -= vf.interLineSpace+vf.height+1;
  sprintf(s,"%5.2f fps (%c)",frameRate,mode);
  drawVString(&vf,s,m,&c,VF_ALIGN_CENTER,color);

} // displayInfo

static void initSMC(long stereo,coord msep,coord sep,coord zoom) {

  smcl.vmag.x = zoom;
  if (stereo) {
    smcl.msep       = msep;
    smcl.center.x   = 159 - sep;
    smcl.vmag.y     = MUL(zoom,TOFIXED(10./11.));
    smcl.center.y   =  99;
    smcl.clip.max.y = 199;
  } else {
    smcl.msep       = 0;
    smcl.center.x   = 159;
    smcl.vmag.y     = MUL(zoom,TOFIXED(10./11.));
    smcl.center.y   =  99;
    smcl.clip.max.y = 199;
  } // if
  smcl.clip.min.x = 0;
  smcl.clip.min.y = 0;
  smcl.clip.max.x = 319;
  smcl.clip.min.z = TOFIXED(2.);
  smcl.clip.max.z = TOFIXED(2500.);
  smcr = smcl;
  smcr.msep = -smcr.msep;
  smcr.center.x = 159 + sep;

  computeOffsets();   // vgfx scanline offsets for mono/stereo

// Center for vector font
  vcenter.x = (smcl.clip.max.x - smcl.clip.min.x + 1)/2;
  vcenter.y = smcl.clip.max.y - vf.interLineSpace*4;

} // initSMC

static void getViewMatrix(matrix34 * m,coord size) {
matrix34 x,y,z,t;
static vec3 axisX = {ONE,0,0};
static vec3 axisY = {0,ONE,0};
static vec3 axisZ = {0,0,ONE};
static long angle = 0;

  if (autoRot) {
    matrix33axis(&x.m,&axisX,angle);
    matrix33axis(&y.m,&axisY,angle);

    matrix33axis(&z.m,&axisZ,angle += 200);
  } else {
    matrix33axis(&x.m,&axisX,ji.js[0].x*10);
    matrix33axis(&y.m,&axisY,ji.js[0].y*10);
    matrix33axis(&z.m,&axisZ,ji.js[1].x*10);
  } // if

  matrixMul33(&t.m,&x.m,&y.m);
  matrixMul33(&m->m,&t.m,&z.m);

// For a static test image:
//  matrix33unitize(&m->m);

  SETVEC(m->w,0,0,TOFIXED(50.));

} // getViewMatrix

#define DEF_MSEP  27038
#define DEF_SEP   6
#define DEF_ZOOM  200
#define IN_MSEP  28676
#define IN_SEP   18
#define IN_ZOOM  344
#define OUT_MSEP 30314
#define OUT_SEP  3
#define OUT_ZOOM 192

#define NUM_MODES 5

static void demoView(long mode) {
  switch (mode) {
    case 0:
      background = 1;
      sep  = DEF_SEP;
      msep = DEF_MSEP;
      zoom = DEF_ZOOM;
      break;
    case 1:
      background = 1;
      msep = IN_MSEP;
      sep  = IN_SEP;
      zoom = IN_ZOOM;
      break;
    case 2:
      background = 1;
      msep = OUT_MSEP;
      sep  = OUT_SEP;
      zoom = OUT_ZOOM;
      break;
    case 3:
      background = 0;
      sep  = DEF_SEP;
      msep = DEF_MSEP;
      zoom = DEF_ZOOM;
      break;
    case 4:
      background = 0;
      msep = IN_MSEP;
      sep  = IN_SEP;
      zoom = IN_ZOOM;
      break;
    case 5:
      background = 0;
      msep = OUT_MSEP;
      sep  = OUT_SEP;
      zoom = OUT_ZOOM;
      break;
  } // switch
} // demoView

void soundHandler(void) {
// Call sound mixer here...
} // soundHandler

static PageAddresses lcdPages[2] = {
  {0<<14,0,  // page 0 Left
   1<<14,0}, // page 0 Right
  {2<<14,0,  // page 1 Left
   3<<14,0}  // page 1 Right
};

void gridDemo(portInfo * pi,long lines,coord size,long useLcdbios) {
float vrr;
matrix34 m,vm;
char * dram;
long joyPolled = 0;
long bcolor    = 0;
long quit      = 0;
long inAndOut  = 0;
long intMask[2];
float frameRate;
int majV,minV;

  if (useLcdbios) {
    if (!isLCDBiosInstalled(&majV,&minV)) {
      printf("LCDBios not installed.");
      return;
    } // if
  } // if

// Set masks of interrupts allowed to run while reading joysticks
  if (useLcdbios) {
    intMask[0] = INT_MASK_TIMER_0 | INT_MASK_2ND_PIC;
    intMask[1] = INT_MASK_RTC;
  } else {
    intMask[0] = INT_MASK_TIMER_0;
    intMask[1] = 0;
  } // if

  initJoystick(JOY_A|JOY_B,MY_JOY_MAX_READ,intMask[0],intMask[1]);

  if (!loadVFont("simple.vft",&vf)) {
    printf("Couldn't load font: simple.vft.\n");
    exit(0);
  } // if

//  scaleVFont(&vf,TOFIXED(1.5),TOFIXED(6.),TOFIXED(1.));

  dram = calloc(320,200*2);
  if (!dram) {
    printf("Not enough memory for drawing buffer.\n");
    exit(0);
  } // if

  d_setDramPages(dram);

  if (!initGrid(lines,size)) {
    printf("Not enough memory for grid.\n");
    exit(0);
  } // if

  matrix33unitize(&vm.m);

// ************* Graphics, timer, and LCD setup *******************

// Initialize the VGA graphics hardware
  initXmode();

// Get the Vertical Refresh Rate (in Hz)
  vrr = getVRR();

printf("vrr %f\n",vrr);

// Init the timer system and LCD driver
  initLCDsys(vrr,
             TIMER_ADVANCE_PERCENT,
             VRR_MULTIPLIER,
             SOUND_DRIVER_CALL_RATE,
             gfxPageHandler,
             soundHandler,
             pi);

// Set the rendering mode and initialize the graphics system
  if (useLcdbios) {
    stereoMode = SM_LCDB;
    setGfxRender(GFX_RENDER_DRAM_LCDBIOS);
// Turn on the video display system and start flipping the LCD shutters
    LCDBiosInit3DMode(TwoBuffer,2,lcdPages);
  } else {
    stereoMode = SM_VRAM;
    setGfxRender(GFX_RENDER_VRAM);
// Turn on the video display system and start flipping the LCD shutters
    vDisplayOn();
    LCDsOn();
  } // if

// Set the display format to stereo
  (*gfx.setDisplayMode)(DISPLAY_STEREO);

// ****************************************************************

  demoView(0);
  initSMC(gfx.dispMode == DISPLAY_STEREO,msep,sep,zoom);

  quit = 0;
  while (!quit) {

    if (kbhit()) {
      char ch = getch();
      ch = toupper(ch);
      switch (ch) {
        case 0x1b: // ESC
        case 'Q': quit = 1; break;
        case 'A': autoRot ^=1; break;
        case '\\':
          if (++inAndOut > NUM_MODES) inAndOut = 0;
          demoView(inAndOut);
          break;
        case 'J': showJoyInfo ^= 1; break;
        case 'T': joyPolled ^= 1; break;
        case '7': sep -= 1; break;
        case '8': sep += 1; break;
        case '9': msep -= TOFIXED(.1); break;
        case '0': msep += TOFIXED(.1); break;
        case '-': zoom -= TOFIXED(.0005); break;
        case '=': zoom += TOFIXED(.0005); break;
        case 'S':
          if (gfx.dispMode == DISPLAY_STEREO) {
            (*gfx.setDisplayMode)(DISPLAY_MONO);
            clearLCDs();
          } else {
            (*gfx.setDisplayMode)(DISPLAY_STEREO);
          } // if
          break;
        case 'D':
          if (useLcdbios && stereoMode == SM_LCDB) {
            LCDBiosDeinit3DMode();
          } // if
          if (stereoMode == SM_DRAM) {
            LCDsOff();
            vDisplayOff();
          } // if
          waitVS();
          if (++stereoMode > SM_LCDB) stereoMode = SM_VRAM;
          if (!useLcdbios && stereoMode == SM_LCDB) stereoMode = SM_VRAM;
          switch (stereoMode) {
            case SM_VRAM: setGfxRender(GFX_RENDER_VRAM);
              LCDsOn();
              vDisplayOn();
              break;
            case SM_DRAM: setGfxRender(GFX_RENDER_DRAM);
              LCDsOn();
              vDisplayOn();
              break;
            case SM_LCDB: setGfxRender(GFX_RENDER_DRAM_LCDBIOS);
              LCDBiosInit3DMode(TwoBuffer,2,lcdPages);
              break;
          } // switch
          break;
        case 'B': background ^= 1;
          if (background) bcolor = DAC(2,2,1); else bcolor = 0; break;
        case 'P': while (!kbhit()); getch(); break;
      } // switch
      initSMC(gfx.dispMode == DISPLAY_STEREO,msep,sep,zoom);
    } // if

    if (showJoyInfo) {
      if (joyPolled)
        joyReadOk = readJoystick(&ji);      // Polled
      else
        joyReadOk = readJoystickTimer(&ji); // Timed
    } // if

    if (ji.js[0].button[0]) {
      zoom -= TOFIXED(.0005);
      initSMC(gfx.dispMode == DISPLAY_STEREO,msep,sep,zoom);
    } // if

    if (ji.js[0].button[1]) {
      zoom += TOFIXED(.0005);
      initSMC(gfx.dispMode == DISPLAY_STEREO,msep,sep,zoom);
    } // if

    getViewMatrix(&m,size);

    if (background) bcolor = DAC(2,2,1); else bcolor = 0;

    (*gfx.waitView)();

    (*gfx.setDrawPage)(DRAW_LEFT);
    (*gfx.clearPage)(bcolor);
    if (gfx.dispMode == DISPLAY_STEREO) {
      (*gfx.setDrawPage)(DRAW_RIGHT);
      (*gfx.clearPage)(bcolor);
    } // if

    if (gfx.dispMode == DISPLAY_STEREO) {
      setStereoSMC(&smcl);
      (*gfx.setDrawPage)(DRAW_LEFT);
      drawGrid(&m);
      displayInfo(0,&vcenter,frameRate);
      setStereoSMC(&smcr);
      (*gfx.setDrawPage)(DRAW_RIGHT);
      drawGrid(&m);
      displayInfo(0,&vcenter,frameRate);
    } else {
      setStereoSMC(&smcl);
      (*gfx.setDrawPage)(DRAW_LEFT);
      drawGrid(&m);
      displayInfo(0,&vcenter,frameRate);
    } // if

    (*gfx.updateView)();

    frameRate = getFrameRate();

  } // while

  deInitLCDsys();

  if (useLcdbios && stereoMode == SM_LCDB) LCDBiosDeinit3DMode();

  deInitXmode();

  deInitGrid();

  free(dram);

  freeVFontMembers(&vf);

} // gridDemo

