// joystick.h
// Copyright 1995 John Schultz
// All Rights Reserved

#ifndef JOYSTICK_H
#define JOYSTICK_H

// In readjoy.asm:
#pragma aux readJoy "_*" \
  parm [EDI] modify [EBX ECX EDX];
int readJoy(int flags);

extern long	joyX1;
extern long	joyY1;
extern long	joyX2;
extern long	joyY2;
extern long joyButtons;

// In joystick.c:

// Set this flag to 1 in all interrupt code.
extern volatile long interrupted;

typedef struct {
  long x,y;
  short button[2];
} joystick;

typedef struct {
  joystick js[2];
} joyInfo;

// Bit flags for joyflags var.
// Do not read a joystick axis with no joystick (pot) attached:
// this will waste cycles.
#define JOY_A_X     (1<<0)
#define JOY_A_Y     (1<<1)
#define JOY_B_X     (1<<2)
#define JOY_B_Y     (1<<3)
#define JOY_A       (JOY_A_X|JOY_A_Y)
#define JOY_B       (JOY_B_X|JOY_B_Y)
#define JOY_MASK    (JOY_A|JOY_B)

#define INT_MASK_TIMER_0  (1<<0)
#define INT_MASK_KEYBOARD (1<<1)
#define INT_MASK_2ND_PIC  (1<<2)
#define INT_MASK_SER_2    (1<<3)
#define INT_MASK_SER_1    (1<<4)
#define INT_MASK_PAR_2    (1<<5)
#define INT_MASK_DISK     (1<<6)
#define INT_MASK_PAR_1    (1<<7)

// 2nd PIC
#define INT_MASK_RTC      (1<<0)

// initJoystick(JOY_A | JOY_B_Y,2048,INT_MASK_TIMER_0 | INT_MASK_SER_2,0);
// Above is an example initJoyStick() call which:
// reads a 3 axis joystick (x, y, throttle), has a max read count of 2048,
// and allows the timer and COM2 interrupts to continue running.
// The timer and serial interrupt handlers must set the variable "interrupted"
// to 1 everytime they are called.
// intMask2 is for the cascaded PIC, and is used with LCDBIOS (IRQ8).
void initJoystick(long flags,long joyMaxRead,long intMaskSet1,long intMaskSet2);

// Returns true if joystick was read successfully without being interrupted,
// and no pot exceeded joyMaxRead set in initJoystick.
long  readJoystick(joyInfo * ji);

// Reads joystick using microsecond timer in vtimer.c.
// initLCDsys() must be called before this routine will work.
long  readJoystickTimer(joyInfo * ji);

#endif

// joystick.h

