/* test.c */
/* Created 1/15/95 */
/* John Schultz */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include "imath.h"
#include "vector.h"
#include "vtimer.h"
#include "gridemo.h"
#include "option.h"

void showInfo(char * prog) {
  printf("\n\n\nUsage:\n");
  printf("  %s [-ioPort] [-lines #] [-size #] [-lcdbios]\n",prog);
  printf("    -ioPort  : LPT1,LPT2,LPT3,LPT4,COM1,COM2,COM3,COM4.\n");
  printf("    -lines # : Number of lines per cube face (2-256).\n");
  printf("    -size #  : Cube size (1-2000) [Affects stereo 3D].\n");
  printf("    -lcdbios : Use LCDBIOS to page flip. Press D to cycle modes.\n");
  printf("Example:\n  %s -LPT1 -lines 16 -size 10\n",prog);
  printf("The default values are: LPT1, lines = 12, size = 20\n");
  printf("The following keys are used while grid is running:\n");
  printf("    s        : Toggle Stereoscopic 3D on and off.\n");
  printf("    t        : Toggle Timed or Polled joystick reading.\n");
  printf("    a        : Toggle auto rotate or joystick rotate.\n");
  printf("    -,=      : Zoom out, zoom in view.\n");
  printf("    7,8      : Decrease, increase stereo separation.\n");
  printf("    9,0      : Decrease, increase stereo interocular angle.\n");
  printf("    \\        : Reset view to default.\n");
  printf("    b        : Toggle Background color.\n");
  printf("    d        : Toggle DRAM offscreen rendering on and off.\n");
  printf("    v        : Toggle video sync on and off.\n");
  printf("    q,ESC    : Quit.\n");
  exit(0);
} // showInfo

void main(int argc,char ** argv) {
portInfo pi;
long lines;
coord size;
long useLcdbios;
long i;

  initOpt(argc,argv);

  useLcdbios = getOpt("-lcdbios");

  if (getOpt("-?") || getOpt("/?") || getOpt("?") ||
    getOpt("-h") || getOpt("/h") || getOpt("h")) {
    showInfo(argv[0]);
  } // if

  if (i=getOpt("-lines")) {
    if (!sscanf(argv[i+1],"%ld",&lines)) {
      printf("A number must be specified after the -lines argument.\n");
      printf("For example: -lines 12\n");
      exit(1);
    } // if
    CLAMPMINMAX(lines,2,256);
  } else {
    lines = 12;
  } // if

  if (i=getOpt("-size")) {
    if (!sscanf(argv[i+1],"%ld",&size)) {
      printf("A number must be specified after the -size argument.\n");
      printf("For example: -size 20\n");
      exit(1);
    } // if
    CLAMPMINMAX(size,1,2000);
  } else {
    size = 20;
  } // if

  if (getOpt("/LPT1")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_LPT1)) {
      printf("\nPort %s not found.\n","LPT1");
      exit(1);
    } // if
  } else if (getOpt("/LPT2")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_LPT2)) {
      printf("\nPort %s not found.\n","LPT2");
      exit(1);
    } // if
  } else if (getOpt("/LPT3")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_LPT3)) {
      printf("\nPort %s not found.\n","LPT3");
      exit(1);
    } // if
  } else if (getOpt("/LPT4")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_LPT4)) {
      printf("\nPort %s not found.\n","LPT4");
      exit(1);
    } // if
  } else if (getOpt("/COM1")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_COM1)) {
      printf("\nPort %s not found.\n","COM1");
      exit(1);
    } // if
  } else if (getOpt("/COM2")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_COM2)) {
      printf("\nPort %s not found.\n","COM2");
      exit(1);
    } // if
  } else if (getOpt("/COM3")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_COM3)) {
      printf("\nPort %s not found.\n","COM3");
      exit(1);
    } // if
  } else if (getOpt("/COM4")) {
    if (!getPortInfo(&pi,STEREO3D_PORT_COM4)) {
      printf("\nPort %s not found.\n","COM4");
      exit(1);
    } // if
  } else {
    if (!getPortInfo(&pi,STEREO3D_PORT_LPT1)) {
      printf("\nPort %s not found.\n","LPT1");
      printf("An I/O port for the CyberShades must be specified.\n");
      showInfo(argv[0]);
    } // if
  } // if

  gridDemo(&pi,lines,SHL(size),useLcdbios);

} // main
