/* vgfx.c */
/* Created 4/3/94, modified 1/11/95 */
/* Copyright 1995 John Schultz */
/* All Rights Reserved */

#include <math.h>
#include <conio.h>
#include <dos.h>

#include "imath.h"
#include "vector.h"
#include "vtimer.h"
#include "gfx.h"

#ifdef __WATCOMC__
#define USE_ASM_VUTIL
#define FAR
#define FAR_P(p) ((void *)(p))
#else
#define FAR __far
#define FAR_P(p) ((void FAR *)MK_FP((ushort)((ulong)(p)>>4),(ushort)((ulong)(p)&0xfL)))
#endif

#define GAMMA 1.2

#define ATTR_CTRL  0x03C0
#define PAL_INDEX  0x03C8
#define PAL_WRITE  0x03C9
#define INP_STAT_1 0x03DA
#define PAS_SET    (1<<5)
#define OVRSCN_CLR 0x0011
#define SC_INDEX   0x03C4
#define GC_INDEX   0x03CE
#define CRTC_INDEX 0x03D4
// Low byte is index, high byte is value:
#define CHAIN4_OFF 0x0604
#define DWORD_OFF  0x0014
#define BWORD_ON   0xE317
#define LATCH_BITS 0x0008
#define CPU_BITS   0xFF08
#define PLANE_MASK 0x0F02
#define VSYNC_MASK (1<<3)

// Color latch mode and normal cpu memory access
#define LATCH_MODE outpw(GC_INDEX,LATCH_BITS); outpw(SC_INDEX,PLANE_MASK)
#define CPU_MODE   outpw(GC_INDEX,CPU_BITS);   outpw(SC_INDEX,PLANE_MASK)

// Video memory page 0, flat 32-bit address
#define VMPAGE0 0xa0000L

ulong vmpage[4] = {VMPAGE0,VMPAGE0+(1<<14),VMPAGE0+(2<<14),VMPAGE0+(3<<14)};
static char planeMask[320];
static short yOffset[200];
static long gcolors[256];
static short oldVmode=0;
static long drawPage=0;

// Intensity Range, Intensity Max Value
#define IRANGE 64
#define IMAX ((float)(IRANGE-1))
static unsigned char gtab[IRANGE];

// 1.0 = no gamma correction
void compGamma(float intensity) {
long i;
  CLAMPMIN(intensity,.01);
  for (i=0; i < IRANGE ; i++) {
    gtab[i] = IMAX*pow(i/IMAX,1./intensity) + .5;
  } // for
} // compGamma

void setPalette(long * colors,long count,float gamma) {
long i;
  compGamma(gamma);
  CLAMPMAX(count,256);
  outp(0x3C8,0);   // Reset write color palette index
  for (i=0; i < count; i++) {
    unsigned char r,g,b;
    r = colors[i] & 0xff;
    r = gtab[r];
    outp(0x3C9,r); // Write to color palette index: it autoincrements
    g = (colors[i] >> 8) & 0xff;
    g = gtab[g];
    outp(0x3C9,g); // Write to color palette index: it autoincrements
    b = (colors[i] >> 16) & 0xff;
    b = gtab[b];
    outp(0x3C9,b); // Write to color palette index: it autoincrements
  } // for
} // setPalette

void setGamma(float gamma) {
  setPalette(gcolors,256,gamma);
} // setGamma

void setOverscanColor(long color) {
  inp(INP_STAT_1);
  outp(ATTR_CTRL,OVRSCN_CLR);
  outp(ATTR_CTRL,color);
  inp(INP_STAT_1);
  outp(ATTR_CTRL,PAS_SET);
} // setOverscanColor

void setVmode(long mode) {
union REGS regs;
  oldVmode  = *(short *)0x449; // Current video mode
  regs.h.al = mode;
  regs.h.ah = 0;               // function number
#ifdef __WATCOMC__
  int386(0x10,&regs,&regs);
#else
  int86(0x10,&regs,&regs);
#endif
} // setVmode

void restoreVmode(void) {
  if (oldVmode) {
    setVmode(oldVmode);
    oldVmode = 0;
  } // if
} // restoreVmode

void computeOffsets(void) {
long i;
  for (i=0; i < 200; i++) yOffset[i] = i*80;
} // computeOffsets

#define VMODE_320x200_256 0x13

void initXmode(void) {
long r,g,b;
long i,j;
long FAR * addr;

  setVmode(VMODE_320x200_256);

  outp(0x3c2,inp(0x3cc)|4); // Increase vertical refresh rate

// Set up an unchained display (Mode X)
  outpw(SC_INDEX,CHAIN4_OFF);
  outpw(CRTC_INDEX,DWORD_OFF);
  outpw(CRTC_INDEX,BWORD_ON);

  CPU_MODE;

  addr = (long FAR *)FAR_P(VMPAGE0);
  for (i=0; i < (1<<14); i++) { /* clear all pages */
    addr[i] = 0;
  } // for

  for (i=0; i < 320; i++) planeMask[i] = (1<<(i&3));
  computeOffsets();

  for (i=0; i < 256; i++) {
    long r,g,b;
    r = (i&7)*63/7;
    g = ((i>>3)&7)*63/7;
    b = (i>>6)*63/3;
    gcolors[i] = (b<<16)|(g<<8)|r;
  } // for

  setPalette(gcolors,256,GAMMA);

  setOverscanColor(0);

} // initXmode

void deInitXmode(void) {
  restoreVmode();
} // deInitXmode

void setViewPageAddr(ulong addr) {
#ifdef USE_ASM_VUTIL
  set_vpage_adr(addr);
#else
unsigned short a,b;
  b = (addr & 0xff00) | 0xc;
  a = ((addr << 8) & 0xff00) | 0xd;
  outpw(CRTC_INDEX,a);
  outpw(CRTC_INDEX,b);
#endif
} // setViewPageAddr

void waitVS(void) {
#ifdef USE_ASM_VUTIL
  vsync();
#else
/* If active, wait for inactive */
  if (inp(INP_STAT_1) & VSYNC_MASK) {
/* Wait for inactive */
	while (1) {
	  if (!(inp(INP_STAT_1) & VSYNC_MASK)) break;
	} // while
  } // if
/* Wait for active */
  while (1) {
	if ((inp(INP_STAT_1) & VSYNC_MASK)) break;
  } // while
#endif
} // waitVS

void v_setDisplayMode(displayMode dm) {
  _disable();
  vdata.view = 0;
  gfx.advancePage = 0;
  gfx.dispMode = dm;
  if (gfx.dispMode == DISPLAY_STEREO) {
    gfx.viewPage = 0;
    gfx.drawPage = 2;
  } else {
    gfx.viewPage = 0;
    gfx.drawPage = 1;
  } // if
  gfx.numPages   = 4;
  drawPage = gfx.drawPage;
  _enable();
} // v_setDisplayMode

void v_clearPage(long color) {
long FAR * addr;
long i;
  outpw(SC_INDEX,PLANE_MASK); /* Write to all planes */
  color = (color<<24)|(color<<16)|(color<<8)|color;
  addr = (long FAR *)FAR_P(vmpage[drawPage]);
  for (i=0; i < 16000/4/8; i++) {
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
    *addr++ = color;
  } /* for */
} // v_clearPage

void v_setPixel(vec2 * p,long color) {
  outpw(SC_INDEX,(planeMask[p->x]<<8)|2);
  *(char FAR *)FAR_P(vmpage[drawPage] + (p->x>>2) + yOffset[p->y]) = color;
} // v_setPixel

void v_drawLine(vec2 * l,long color) {
long x1 = l[0].x;
long y1 = l[0].y;
long x2 = l[1].x;
long y2 = l[1].y;
long d,x,y,ax,ay,sx,sy,dx,dy;
char FAR * baddr = FAR_P(vmpage[drawPage]);
char FAR * addr;
  outp(SC_INDEX,2); /* Set up (point to) plane mask register */
  dx = x2-x1; ax = abs(dx)<<1; sx = sgn(dx);
  dy = y2-y1; ay = abs(dy)<<1; sy = sgn(dy);
  x = x1;
  y = y1;
  if (ax > ay) { /* x dominant */
    d = ay - (ax>>1);
    addr = baddr + yOffset[y];
    while (1) {
      outp(SC_INDEX+1,planeMask[x]);
      addr[x>>2] = color;
	     if (x == x2) return;
	     if (d >= 0) {
	       y += sy; d -= ax;
         addr = baddr + yOffset[y];
	     } // if
	     x += sx; d += ay;
    } // while
  } else {       /* y dominant */
    d = ax - (ay>>1);
    addr = baddr + (x>>2);
    outp(SC_INDEX+1,planeMask[x]);
    while (1) {
	     addr[yOffset[y]] = color;
	     if (y == y2) return;
	     if (d >= 0) {
	       x += sx; d -= ay;
	       outp(SC_INDEX+1,planeMask[x]);
	       addr = baddr + (x>>2);
	     } // if
	     y += sy; d += ax;
	   } // while
  } // if
} // v_drawLine

void v_fillRect(vec2 * extent,long color) {
long x1 = extent[0].x;
long y1 = extent[0].y;
long x2 = extent[1].x;
long y2 = extent[1].y;
char FAR * baddr = FAR_P(vmpage[drawPage]);
char FAR * addr;
long x,y,temp;
  if (x1 > x2) {temp = x1; x1 = x2; x2 = temp;}
  if (y1 > y2) {temp = y1; x1 = y2; y2 = temp;}
  outp(SC_INDEX,2); /* Set up (point to) plane mask register */
  for (x=x1; x <= x2; x++) {
	   outp(SC_INDEX+1,planeMask[x]);
	   addr = baddr + (x>>2);
	   for (y=y1; y <= y2; y++) {
	     addr[yOffset[y]] = color;
	   } // for
  } // for
} // fillRect

void v_waitView(void) {
  if (gfx.dispMode == DISPLAY_STEREO) {
    while (gfx.advancePage);
  } // if
}  // v_waitView

void v_updateView(void) {
  while (gfx.advancePage);
  if (gfx.dispMode == DISPLAY_STEREO) {
    if ((gfx.drawPage += 2) >= gfx.numPages) gfx.drawPage = 0;
  } else {
    if ((gfx.drawPage += 1) >= gfx.numPages) gfx.drawPage = 0;
  } // if
  drawPage = gfx.drawPage;
  vdata.simFrames++;
  gfx.advancePage = 1;
} // v_updateView

void v_setViewPage(long page) {
  setViewPageAddr(vmpage[page]);
} // v_setViewPage

void v_setDrawPage(gfxDrawLR dlr) {
  if (dlr == DRAW_LEFT) {
    drawPage = gfx.drawPage;
  } else {
    drawPage = gfx.drawPage+1;
  } // if
} // v_setDrawPage

// vgfx.c
