// vri.c
// Created 10/14/95
// Vertical Retrace Interrupt checking code
// Copyright 1995 John Schultz
// All Rights Reserved

#include <stdio.h>
#include <conio.h>
#include "vri.h"

//#define TEST

#define IS0_REG 0x3c2   // Input Status 0 register
#define VRI_BIT (1<<7)  // Vertical Retrace Interrupt bit

#define CRTC_REG 0x3d4  // CRT Controller address register
#define VRE_ADDR  0x11  // Vertical Retrace End register (CRTC address offset)
#define CVI_BIT  (1<<4) // Clear Vertical Interrupt bit
#define DVI_BIT  (1<<5) // Disable Vertical Interrupt bit
#define CVI_DVI_MASK (CVI_BIT|DVI_BIT)

static long origVRI,oldVRI,setVRI[2]={0,0};

long checkVRI(void) {
  return (long)(inp(IS0_REG) & VRI_BIT) != 0;
} // checkVRI

long readVRI(void) {
  outp(CRTC_REG,VRE_ADDR);
  return (long)inp(CRTC_REG);
} // readVRI

void writeVRI(long val) {
  outpw(CRTC_REG,(val << 8) | VRE_ADDR);
} // writeVRI

void getVRI(void) {
  oldVRI = ((readVRI() & ~CVI_DVI_MASK) << 8) | VRE_ADDR;
  setVRI[0] = oldVRI | (DVI_BIT<<8);
  setVRI[1] = oldVRI | (CVI_BIT<<8);
} // getVRI

void resetVRI(void) {
  getVRI(); // Needed if display mode changes after initVRI().
  outpw(CRTC_REG,setVRI[0]);
  outpw(CRTC_REG,setVRI[1]);
} // resetVRI

void initVRI(void) {
  origVRI = readVRI();
//  getVRI(); // Needed if display mode doesn't change after initVRI().
  resetVRI();
} // initVRI

void restoreVRI(void) {
  writeVRI((readVRI() & ~CVI_DVI_MASK) | (origVRI & CVI_DVI_MASK));
} // restoreVRI

#ifdef TEST

void main() {
  initVRI();

  while (!kbhit()) {
    long vri = checkVRI();
    printf("VRI %d\n",vri);
    if (vri) resetVRI();
  } // while

  getch();

  restoreVRI();

} // main

#endif
