// vtimer.c
// Created 1/7/94
// Copyright 1995 John Schultz
// All Rights Reserved

#include <stdio.h>
#include <conio.h>
#include <dos.h>

#include "imath.h"
#include "vtimer.h"
#include "joystick.h"
#include "isrstub.h"
#include "vri.h"

#define USE_LOCAL_STACK

volatile vTimerData vdata;

#define SYSTEM_CLOCK 1193180 // 1.19318 MHz
#define SYSTEM_CLOCK_TO_MICROS (1e6/(float)SYSTEM_CLOCK)

#define TIMER_VECTOR     0x8 // IRQ0 timer 0, vector 0x8

#define TIMER_CONTROL   0x43 // Timer control register port
#define TIMER_MODE      0x34 // Mode setup command for timer
#define TIMER_PORT_0    0x40 // Hardware timer 0 port
#define TIMER_LATCH     0x00 // Timer command to latch in count

#define PIC_CMD         0x20 // Proc. Int. Control
#define NON_SPEC_EOI    0x20 // Non Specific End Of Interrupt
#define PIC_MASK        0x21 // Interrupt mask register
#define IRQ0_MASK     (1<<0) // Timer 0 mask

#define INPUT_STATUS_1 0x3da // VGA register
#define VSYNC_MASK       0x8 // Active high

void (__interrupt __far * prevTimerInt)() = 0;

// Serial port display bits
#define SER_D_LEFT  0x1
#define SER_D_RIGHT 0x3
#define SER_D_OFF   0x0

// Parallel port display bits
#define PAR_D_LEFT  (0xff)
#define PAR_D_RIGHT (~(1<<2))
#define PAR_D_OFF   0x0

#define COM_OFFSET 0x4 // Offset to COM I/O pins

static ushort * serPort = (ushort *)0x400; // Low memory address tables
static ushort * parPort = (ushort *)0x408;

// Set up timer latches and reset count
void setTimer(long timerInit) {
  outp(TIMER_CONTROL,TIMER_MODE);
  outp(TIMER_PORT_0,timerInit&0xff); // low
  outp(TIMER_PORT_0,timerInit>>8);   // high
} // setTimer

long getTimer(void) {
long timerLo,timerHi;
  outp(TIMER_CONTROL,TIMER_LATCH);
  timerLo = inp(TIMER_PORT_0);
  timerHi = inp(TIMER_PORT_0);
  return (timerHi << 8) | timerLo;
} // getTimer

long getMicroSecs(void) {
unsigned long timerLo,timerHi;
unsigned long mcs;
  _disable();
  outp(TIMER_CONTROL,TIMER_LATCH);
  timerLo = inp(TIMER_PORT_0);
  timerHi = inp(TIMER_PORT_0);
  mcs = (timerHi << 8) | timerLo;
// The +1 below is due to interrupts firing when the count is
// is -1, so the total microseconds per interrupt are timerInit+1.
  mcs = (vdata.timerInit - mcs) + (vdata.timerInit+1)*vdata.intCount + vdata.busyWaitCount;
  mcs = mcs*SYSTEM_CLOCK_TO_MICROS;
  _enable();
  return (long)mcs;
} // getMicroSecs

void resetMicroSecs(void) {
  _disable();
  vdata.intCount = 0;
  vdata.busyWaitCount = 0;
  _enable();
} // resetMicroSecs

long checkSim18_2(void) {

  if (vdata.readingJoystick) return 0;

  vdata.sim18_2 += vdata.deltaSim18_2;
  if (vdata.sim18_2 >= TOFIXED(1.)) {
    vdata.sim18_2 -= TOFIXED(1.); // keep fractional bits
    return 1;
  } else {
    return 0;
  } // if
} // checkSim18_2

#if 0
static void dcolor(void) {
long dr,dg,db;
  dr = rand();
  dg = rand();
  db = rand();
  outp(0x3c8,0);  // Set random background colors.
  outp(0x3c9,dr);
  outp(0x3c9,dg);
  outp(0x3c9,db);
} // dcolor
#endif

void sim18_2wEOI(void) {

  if (checkSim18_2()) {
    (*prevTimerInt)();
  } else {
    outp(PIC_CMD,NON_SPEC_EOI);
  } // if
} // sim18_2wEOI

static void displayHandler(void) {

// Flip LCD shutters and display pages
  vdata.view ^= 1;

// Update display pages
  (*vdata.displayPageFunc)(vdata.view);

// Synchronize to vertical retrace

  while (!checkVRI()) if (inp(INPUT_STATUS_1) & VSYNC_MASK) break;
  resetVRI();
  vdata.busyWaitCount += vdata.timerInit - getTimer();
  setTimer(vdata.timerInit);

// Flip LCD shutters
  if (vdata.flipLCDs && vdata.VSport) {
    if (vdata.view) {
      outp(vdata.VSport,vdata.VSright);
    } else {
      outp(vdata.VSport,vdata.VSleft);
    } // if
  } // if

} // displayHandler

// isrHandler() is called by isrStub() in isrstub.asm, or called
// by vsHandler() below (depending on the USE_LOCAL_STACK define).

void isrHandler(void) {

  if (--vdata.displayCounter <= 0) {
    if (vdata.displayOn) displayHandler();
    vdata.displayCounter = vdata.resetDisplayCounter;
  } // if

  if (!vdata.readingJoystick) {
    vdata.soundCounter += vdata.deltaSoundCounter;
    if (vdata.soundCounter >= TOFIXED(1.)) {
      vdata.soundCounter -= TOFIXED(1.); // keep fractional bits
      if (vdata.soundFunc) (*vdata.soundFunc)();
    } // if
  } // if

#if 0
  if (--vdata.frameTics <= 0) {
    vdata.frameTics = vdata.ticsPerSec;
    vdata.frameRate = vdata.simFrames;
    vdata.simFrames = 0;
  } // if
#endif

  interrupted = 1;  // tell readJoystick() that it was interrupted

  vdata.intCount++; // Update interrupt count (used in getMicroSecs())

} // isrHandler

// vsHandler() is used when no local stack is needed and is installed as
// the int 8 interrupt handler instead of isrStub().

#ifndef USE_LOCAL_STACK

static void __interrupt __far vsHandler(void) {
static long semaLock=0;
static long do18_2;

  if (semaLock) {
    outp(PIC_CMD,NON_SPEC_EOI);
    return;
  } else {
    semaLock = 1;
    do18_2 = checkSim18_2();
    if (!do18_2) outp(PIC_CMD,NON_SPEC_EOI);
  } // if

  _enable();
  isrHandler();
  _disable();

  if (do18_2) {
    (*prevTimerInt)();
    _disable();
  } // if

  semaLock = 0;

} // vsHandler

#endif

// Get the Vertical Refresh Rate

float getVRR(void) {
float Hz;
long count;

  _disable();

// Get system synchronized for subsequent waitVertSyncs
  waitVS();

// Set up timer latches and reset count
  setTimer(0);

// Now wait two vertical retraces
  waitVS();
  waitVS();

  count = getTimer();

  _enable();

  count = (65536 - count)/2;

  Hz = (float)SYSTEM_CLOCK/(float)count; // Get VRR in Hz

  return Hz;

} // getVRR

// Get stereo3D port info
long getPortInfo(portInfo * pi,stereo3Dport sp) {

  if (sp < STEREO3D_PORT_COM1) {        // Parallel port
    pi->VSport    = parPort[sp];
    pi->VSleft    = PAR_D_LEFT;
    pi->VSright   = PAR_D_RIGHT;
    pi->VSoff     = PAR_D_OFF;
  } else if (sp < STEREO3D_PORT_NONE) { // Serial port
    pi->VSport    = serPort[sp-STEREO3D_PORT_COM1];
    if (pi->VSport) pi->VSport += COM_OFFSET;
    pi->VSleft    = SER_D_LEFT;
    pi->VSright   = SER_D_RIGHT;
    pi->VSoff     = SER_D_OFF;
  } else {
    pi->VSport    = 0;
    pi->VSleft    = 0;
    pi->VSright   = 0;
    pi->VSoff     = 0;
  } // if

  return pi->VSport != 0;

} // getPortInfo

static long deInitTimer(void) {
  if (prevTimerInt) {
    _disable();
    _dos_setvect(TIMER_VECTOR,prevTimerInt);
    prevTimerInt = 0;
// Set latches for 18.2Hz (65536 mod 65536 = 0)
    setTimer(0);
    _enable();
    return 1;
  } else {
    return 0;
  } // if
} // deInitTimer

// Initialize the LCD shutter timer and display systems
void initLCDsys(float verticalRefreshRate,
                float timerAdvancePercent,
                long vrrMultiplier,
                long soundCallRate,
                dpfunc displayPageFunc,
                sfunc soundFunc,
                portInfo * pi) {
float Hz;
long tr;

  deInitLCDsys();

// Make sure display is off
  vDisplayOff();

// Set up I/O port to drive LCD hardware
  vdata.VSport  = pi->VSport;
  vdata.VSleft  = pi->VSleft;
  vdata.VSright = pi->VSright;
  vdata.VSoff   = pi->VSoff;

  clearLCDs();

// Increase the timer rate to allow calling sound code
  while (1) {
    Hz = verticalRefreshRate*vrrMultiplier;
    if (Hz >= (float)soundCallRate) break;
    vrrMultiplier++;
  } // while

  vdata.resetDisplayCounter = vrrMultiplier;
  vdata.displayCounter      = vdata.resetDisplayCounter;

  vdata.deltaSoundCounter   = TOFIXED((float)soundCallRate/Hz);
  vdata.soundCounter        = 0;

  vdata.deltaSim18_2        = TOFIXED(18.2/Hz);
  vdata.sim18_2             = 0;

  vdata.ticsPerSec          = Hz + .5;
  vdata.frameTics           = vdata.ticsPerSec;

  Hz *= 1. + timerAdvancePercent/100.;
                               // Increase rate to interrupt before vblank.
                               // Since we busy-wait for vblank, the actual
                               // timer rate is lower than Hz by about
                               // TIMER_ADVANCE_PERCENT.

  vdata.timerInit = (float)SYSTEM_CLOCK/Hz; // Convert back to timer init value.
  vdata.view      = 0;

// Set up function pointers
  vdata.displayPageFunc = displayPageFunc;
  vdata.soundFunc       = soundFunc;

  _disable();

  prevTimerInt = _dos_getvect(TIMER_VECTOR);

#ifdef USE_LOCAL_STACK
  _dos_setvect(TIMER_VECTOR,isrStub);
#else
  _dos_setvect(TIMER_VECTOR,vsHandler);
#endif

// Set up timer latches and reset count
  waitVS();
  setTimer(vdata.timerInit);

  vdata.intCount = 0;

// Init vertical retrace interrupt bits.
// We're just checking bits, so the PIC mask won't be touched and
// vertical retrace interrupts won't occur.
  initVRI();

  _enable();

} // initLCDsys

void clearLCDs(void) {
  if (vdata.VSport) {
    outp(vdata.VSport,vdata.VSoff);
  } // if
} // clearLCDs

void vDisplayOn(void) {
  vdata.displayOn = 1;
} // vDisplayOn

void vDisplayOff(void) {
  vdata.displayOn = 0;
} // vDisplayOff

void LCDsOn(void) {
  vdata.flipLCDs = 1;
} // LCDsOn

void LCDsOff(void) {
  vdata.flipLCDs = 0;
  clearLCDs();
} // LCDsOff

void deInitLCDsys(void) {
  if (deInitTimer()) {
    restoreVRI();
    clearLCDs();
  } // if
} // deInitLCDsys

// vtimer.c

