/* vtimer.h
   Created 3/21/91
   Modified 9/5/92
   Modified 12/29/92
   Modified 11/7/95

   Copyright 1995 John Schultz
   All Rights Reserved
*/

#ifndef VTIMER_H
#define VTIMER_H

// PC I/O ports for LCD driver hardware
typedef enum {
  STEREO3D_PORT_LPT1=0,
  STEREO3D_PORT_LPT2,
  STEREO3D_PORT_LPT3,
  STEREO3D_PORT_LPT4,
  STEREO3D_PORT_COM1,
  STEREO3D_PORT_COM2,
  STEREO3D_PORT_COM3,
  STEREO3D_PORT_COM4,
  STEREO3D_PORT_NONE
} stereo3Dport;

typedef struct {
  long VSport;  // View synchronization I/O port
  long VSleft;  // Left/right/off bits patterns to write to I/O port
  long VSright;
  long VSoff;
} portInfo;

// Sound function pointer type
typedef void (*sfunc)(void);

// Display page function pointer type
typedef void (*dpfunc)(long eye);

// Video/timer data
typedef struct {

  long timerInit;
  long intCount;
  long busyWaitCount;
  long ticsPerSec;
  long frameTics;
  long frameRate;
  long simFrames;

  long view;     // Stereo view: 0 = left, 1 = right
  long flipLCDs;
  long displayOn;

  long VSport;   // View Synchronization I/O Port
  long VSleft;
  long VSright;
  long VSoff;

  long sim18_2;
  long deltaSim18_2;
  long displayCounter;
  long resetDisplayCounter;
  long soundCounter;
  long deltaSoundCounter;

  long readingJoystick; // When set to 1, don't call dos tick or sound code

  dpfunc displayPageFunc;
  sfunc soundFunc;

} vTimerData;

extern volatile vTimerData vdata;

// MicroSecond Timer. Timer count will wrap approx. every 33.33 minutes
// unless resetMicroSecs() is called.
long getMicroSecs(void);

void resetMicroSecs(void);

// Get the Vertical Refresh Rate
float getVRR(void);

// Get stereo3D port info
long getPortInfo(portInfo * pi,stereo3Dport sp);

// Initialize the LCD timer and display systems
void initLCDsys(float verticalRefreshRate,
                float timerAdvancePercent,
                long vrrMultiplier,
                long soundCallRate,
                dpfunc displayPageFunc,
                sfunc soundFunc,
                portInfo * pi);

// De-initialize the LCD system
void deInitLCDsys(void);

// Turn the timer based video display system on and off
void vDisplayOn(void);
void vDisplayOff(void);

// Turn the LCD shutter system on and off
void LCDsOn(void);
void LCDsOff(void);

// Set both LCDs clear
void clearLCDs(void);

#endif

// vtimer.h

